@@ -40,6 +40,8 @@ using Banshee.Base;
 using Banshee.MediaEngine;
 using Banshee.Widgets;
 using Banshee.Configuration;
+using Banshee.Sources;
+using Banshee.SmartPlaylist;
 
 public static class PluginModuleEntry
 {
@@ -67,7 +69,8 @@ namespace Banshee.Plugins.NotificationAreaIcon
                 return new string[] {
                     "Sebastian Dr\u00f6ge",
                     "Aaron Bockover",
-                    "Ruben Vermeersch"
+                    "Ruben Vermeersch",
+                    "Gabriel Burt"
                 };
             }
         }
@@ -75,6 +78,8 @@ namespace Banshee.Plugins.NotificationAreaIcon
         private EventBox event_box;
         private NotificationArea notif_area;
         private Menu menu;
+        private bool menu_is_reversed = false;
+		private RatingMenuItem rating_menu_item;
         private ActionGroup actions;
         private uint ui_manager_id;
 
@@ -105,12 +110,28 @@ namespace Banshee.Plugins.NotificationAreaIcon
                     Catalog.GetString("Close"), CloseWindow)
             });
 
+            actions.Add(new ToggleActionEntry [] {
+                new ToggleActionEntry("ToggleNotificationsAction", null,
+                    Catalog.GetString("_Show Notifications"), null,
+                    Catalog.GetString("Show notifications when song changes"), ToggleNotifications, ShowNotifications)
+            });
+
             Globals.ActionManager.UI.InsertActionGroup(actions, 0);
             ui_manager_id = Globals.ActionManager.UI.AddUiFromResource("NotificationAreaIconMenu.xml");      
-            
             menu = (Menu) Globals.ActionManager.UI.GetWidget("/NotificationAreaIconMenu");
-            menu.ShowAll();
-        
+
+            for(int i = 0; i < menu.Children.Length; i++) {
+                if(menu.Children[i].Name == "Next") {
+                    rating_menu_item = new RatingMenuItem();
+                    rating_menu_item.Activated += OnItemRatingActivated;
+                    ToggleRatingMenuSensitive();
+                    menu.Insert(rating_menu_item, i + 2);
+                    break;
+                }
+            }
+
+            menu.Show();
+            
             popup = new TrackInfoPopup();
             PlayerEngineCore.EventChanged += OnPlayerEngineEventChanged;
 
@@ -213,6 +234,11 @@ namespace Banshee.Plugins.NotificationAreaIcon
             ShowHideMainWindow();
         }
 
+        private void ToggleNotifications(object o, EventArgs args)
+        {
+            ShowNotifications = (actions["ToggleNotificationsAction"] as ToggleAction).Active;
+        }
+
         private void ShowHideMainWindow()
         {
             if (InterfaceElements.MainWindow.IsActive) {
@@ -309,23 +335,56 @@ namespace Banshee.Plugins.NotificationAreaIcon
             }
         }
 
-        private void PositionMenu(Menu menu, out int x, out int y, out bool push_in) 
+        private void OnItemRatingActivated(object o, EventArgs args)
         {
-            PositionWidget(menu, out x, out y, 0);
-            push_in = true;
+            if(PlayerEngineCore.CurrentTrack != null) {
+                PlayerEngineCore.CurrentTrack.Rating = (uint)rating_menu_item.Value;
+                PlayerEngineCore.TrackInfoUpdated();
+            }
+        }
+        
+        private void ToggleRatingMenuSensitive() 
+        {
+            if(PlayerEngineCore.CurrentTrack != null && (SourceManager.ActiveSource is LibrarySource || 
+                SourceManager.ActiveSource is PlaylistSource ||
+                SourceManager.ActiveSource is SmartPlaylistSource)) {
+                rating_menu_item.Reset((int)PlayerEngineCore.CurrentTrack.Rating);
+                rating_menu_item.Show();
+            } else {
+                rating_menu_item.Hide();
+            }
         }
 
-        private void PositionWidget(Widget widget, out int x, out int y, int yPadding) 
+        private void PositionMenu(Menu menu, out int x, out int y, out bool push_in) 
+        {
+            bool on_bottom = PositionWidget(menu, out x, out y, 0);
+            push_in = true;
+            
+            if((on_bottom && !menu_is_reversed) || (!on_bottom && menu_is_reversed)) {
+                menu_is_reversed = !menu_is_reversed;
+                Widget [] frozen_children = new Widget[menu.Children.Length - 1];
+                Array.Copy(menu.Children, 1, frozen_children, 0, frozen_children.Length);
+                for(int i = 0; i < frozen_children.Length; i++) {
+                    menu.ReorderChild(frozen_children[i], 0);
+                }
+            }
+        }
+
+        private bool PositionWidget(Widget widget, out int x, out int y, int yPadding) 
         {
             int button_y, panel_width, panel_height;
             Gtk.Requisition requisition = widget.SizeRequest();
             
             event_box.GdkWindow.GetOrigin(out x, out button_y);
             (event_box.Toplevel as Gtk.Window).GetSize(out panel_width, out panel_height);
+            
+            bool on_bottom = button_y + panel_height + requisition.Height >= event_box.Screen.Height;
 
-            y = (button_y + panel_height + requisition.Height >= event_box.Screen.Height) 
+            y = on_bottom
                 ? button_y - requisition.Height - yPadding
                 : button_y + panel_height + yPadding;
+                
+            return on_bottom;
         }
 
         private void OnMouseScroll(object o, ScrollEventArgs args) 
@@ -424,6 +483,7 @@ namespace Banshee.Plugins.NotificationAreaIcon
                     break;
                 case PlayerEngineEvent.StartOfStream:
                 case PlayerEngineEvent.TrackInfoUpdated:
+                    ToggleRatingMenuSensitive();
                     FillPopup();
                     ShowNotification();
                     break;
@@ -431,6 +491,7 @@ namespace Banshee.Plugins.NotificationAreaIcon
                     // only hide the popup when we don't play again after 250ms
                     GLib.Timeout.Add(250, delegate {
                         if (PlayerEngineCore.CurrentState != PlayerEngineState.Playing) {
+                            ToggleRatingMenuSensitive();
                             popup.Duration = 0;
                             popup.Position = 0;
                             can_show_popup = false;
@@ -448,7 +509,10 @@ namespace Banshee.Plugins.NotificationAreaIcon
             popup.Artist = PlayerEngineCore.CurrentTrack.DisplayArtist;
             popup.Album = PlayerEngineCore.CurrentTrack.Album;
             popup.TrackTitle = PlayerEngineCore.CurrentTrack.DisplayTitle;
-            popup.CoverArtFileName = PlayerEngineCore.CurrentTrack.CoverArtFileName;
+            try {
+                popup.CoverArtFileName = PlayerEngineCore.CurrentTrack.CoverArtFileName;
+            } catch {
+            }
             popup.QueueDraw();
             if (!popup.Visible) {
                 PositionPopup();
